package com.taais.biz.controller;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.excel.utils.ExcelUtil;
import com.taais.common.log.annotation.Log;
import com.taais.common.log.enums.BusinessType;
import com.taais.common.web.annotation.RepeatSubmit;
import com.taais.common.web.core.BaseController;
import jakarta.annotation.Resource;
import com.taais.biz.domain.vo.AlgorithmTaskTrackVo;
import com.taais.biz.domain.bo.AlgorithmTaskTrackBo;
import com.taais.biz.service.IAlgorithmTaskTrackService;

import com.taais.common.core.core.page.PageResult;

/**
 * 可辨识性分析总任务Controller
 *
 * @author wangkang
 * 2025-01-06
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/demo/algorithmTaskTrack")
public class AlgorithmTaskTrackController extends BaseController {
    @Resource
    private IAlgorithmTaskTrackService algorithmTaskTrackService;

    /**
     * 查询可辨识性分析总任务列表
     */
    @SaCheckPermission("demo:algorithmTaskTrack:list")
    @GetMapping("/list")
    public CommonResult<PageResult<AlgorithmTaskTrackVo>> list(AlgorithmTaskTrackBo algorithmTaskTrackBo) {
        return CommonResult.success(algorithmTaskTrackService.selectPage(algorithmTaskTrackBo));
    }

    /**
     * 导出可辨识性分析总任务列表
     */
    @SaCheckPermission("demo:algorithmTaskTrack:export")
    @Log(title = "可辨识性分析总任务", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AlgorithmTaskTrackBo algorithmTaskTrackBo) {
        List<AlgorithmTaskTrackVo> list = algorithmTaskTrackService.selectList(algorithmTaskTrackBo);
        ExcelUtil.exportExcel(list, "可辨识性分析总任务", AlgorithmTaskTrackVo.class, response);
    }

    /**
     * 获取可辨识性分析总任务详细信息
     */
    @SaCheckPermission("demo:algorithmTaskTrack:query")
    @GetMapping(value = "/{id}")
    public CommonResult<AlgorithmTaskTrackVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(algorithmTaskTrackService.selectById(id));
    }

    /**
     * 新增可辨识性分析总任务
     */
    @SaCheckPermission("demo:algorithmTaskTrack:add")
    @Log(title = "可辨识性分析总任务", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody AlgorithmTaskTrackBo algorithmTaskTrackBo) {
        boolean inserted = algorithmTaskTrackService.insert(algorithmTaskTrackBo);
        if (!inserted) {
            return CommonResult.fail("新增可辨识性分析总任务记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改可辨识性分析总任务
     */
    @SaCheckPermission("demo:algorithmTaskTrack:edit")
    @Log(title = "可辨识性分析总任务", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody AlgorithmTaskTrackBo algorithmTaskTrackBo) {
        Boolean updated = algorithmTaskTrackService.update(algorithmTaskTrackBo);
        if (!updated) {
            return CommonResult.fail("修改可辨识性分析总任务记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除可辨识性分析总任务
     */
    @SaCheckPermission("demo:algorithmTaskTrack:remove")
    @Log(title = "可辨识性分析总任务", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = algorithmTaskTrackService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除可辨识性分析总任务记录失败!");
        }
        return CommonResult.success();
    }

    @GetMapping("/listSubTask/{id}")
    public CommonResult listSubTask(@PathVariable Long id) {
        return algorithmTaskTrackService.listSubTask(id);
    }
}
