package com.taais.biz.controller;

import java.util.List;

import com.taais.biz.domain.CommonAlgorithmConfig;
import com.taais.biz.domain.bo.CommonAlgorithmConfigBo;
import com.taais.biz.domain.vo.CommonAlgorithmConfigVo;
import com.taais.biz.service.ICommonAlgorithmConfigService;
import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.excel.utils.ExcelUtil;
import com.taais.common.log.annotation.Log;
import com.taais.common.log.enums.BusinessType;
import com.taais.common.web.annotation.RepeatSubmit;
import com.taais.common.web.core.BaseController;
import jakarta.annotation.Resource;

import com.taais.common.core.core.page.PageResult;

/**
 * 【请填写功能名称】Controller
 *
 * @author km
 * 2024-12-23
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/demo/algorithmConfig")
public class CommonAlgorithmConfigController extends BaseController {
    @Resource
    private ICommonAlgorithmConfigService commonAlgorithmConfigService;

    /**
     * 查询【请填写功能名称】列表
     */
    @SaCheckPermission("demo:algorithmConfig:list")
    @GetMapping("/list")
    public CommonResult<PageResult<CommonAlgorithmConfigVo>> list(CommonAlgorithmConfigBo commonAlgorithmConfigBo) {
        return CommonResult.success(commonAlgorithmConfigService.selectPage(commonAlgorithmConfigBo));
    }

    @GetMapping("/getOne")
    public CommonResult<CommonAlgorithmConfig> list(String algorithmName) {
        return CommonResult.success(commonAlgorithmConfigService.getByAlgorithmName(algorithmName));
    }

    /**
     * 导出【请填写功能名称】列表
     */
    @SaCheckPermission("demo:algorithmConfig:export")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CommonAlgorithmConfigBo commonAlgorithmConfigBo) {
        List<CommonAlgorithmConfigVo> list = commonAlgorithmConfigService.selectList(commonAlgorithmConfigBo);
        ExcelUtil.exportExcel(list, "【请填写功能名称】", CommonAlgorithmConfigVo.class, response);
    }

    /**
     * 获取【请填写功能名称】详细信息
     */
    @SaCheckPermission("demo:algorithmConfig:query")
    @GetMapping(value = "/{id}")
    public CommonResult<CommonAlgorithmConfigVo> getInfo(@PathVariable Integer id) {
        return CommonResult.success(commonAlgorithmConfigService.selectById(id));
    }

    /**
     * 新增【请填写功能名称】
     */
    @SaCheckPermission("demo:algorithmConfig:add")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody CommonAlgorithmConfigBo commonAlgorithmConfigBo) {
        boolean inserted = commonAlgorithmConfigService.insert(commonAlgorithmConfigBo);
        if (!inserted) {
            return CommonResult.fail("新增【请填写功能名称】记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改【请填写功能名称】
     */
    @SaCheckPermission("demo:algorithmConfig:edit")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody CommonAlgorithmConfigBo commonAlgorithmConfigBo) {
        Boolean updated = commonAlgorithmConfigService.update(commonAlgorithmConfigBo);
        if (!updated) {
            return CommonResult.fail("修改【请填写功能名称】记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除【请填写功能名称】
     */
    @SaCheckPermission("demo:algorithmConfig:remove")
    @Log(title = "【请填写功能名称】", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Integer[] ids) {
        boolean deleted = commonAlgorithmConfigService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除【请填写功能名称】记录失败!");
        }
        return CommonResult.success();
    }
}
