package com.taais.biz.controller;


import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.mybatisflex.core.query.QueryWrapper;
import com.taais.biz.domain.CommonAlgorithmConfig;
import com.taais.biz.domain.DataAugmentation;
import com.taais.biz.domain.bo.CommonAlgorithmConfigBo;
import com.taais.biz.domain.bo.DataAugmentationBo;
import com.taais.biz.domain.bo.DataAugmentationResultBo;
import com.taais.biz.domain.bo.VideoStableStartResultBo;
import com.taais.biz.domain.dto.Metric;
import com.taais.biz.domain.vo.*;
import com.taais.biz.service.ICommonAlgorithmConfigService;
import com.taais.biz.service.IVideoStableService;
import com.taais.common.core.config.TaaisConfig;
import com.taais.common.core.service.OssService;
import com.taais.system.domain.SysDictData;
import com.taais.system.domain.SysOss;
import com.taais.system.service.ISysDictDataService;
import com.taais.system.service.ISysOssService;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.apache.commons.compress.archivers.ar.ArArchiveEntry;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.excel.utils.ExcelUtil;
import com.taais.common.log.annotation.Log;
import com.taais.common.log.enums.BusinessType;
import com.taais.common.web.annotation.RepeatSubmit;
import com.taais.common.web.core.BaseController;
import jakarta.annotation.Resource;
import com.taais.biz.service.IDataAugmentationService;

import com.taais.common.core.core.page.PageResult;

import static com.taais.biz.service.impl.ToInfraredServiceImpl.readLogContent;

/**
 * 数据增强Controller
 *
 * @author 0
 * 2024-08-30
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/demo/dataAugmentation")
public class DataAugmentationController extends BaseController {
    @Resource
    private IDataAugmentationService dataAugmentationService;
    @Autowired
    private ISysOssService ossService;
    @Autowired
    private ISysDictDataService sysDictDataService;
    @Autowired
    private IVideoStableService videoStableService;

    @Autowired
    private ICommonAlgorithmConfigService commonAlgorithmConfigService;
    @GetMapping("/compare/num/{task_id}")

    public CommonResult getCompareNum(@PathVariable("task_id") Long taskId) {
        return CommonResult.success(dataAugmentationService.getCompareNum(taskId));
    }


    private static Path getImageAtPathIdx(Path inputPath, int idx) throws IOException {
        try (Stream<Path> paths = Files.list(inputPath)) {
            return paths.sorted()
                .skip(idx)
                .filter(path -> !path.toString().toLowerCase().endsWith(".json"))
                .findFirst()
                .orElseThrow(() -> new IllegalArgumentException("Index " + idx + " is out of bounds for the list of files in " + inputPath));
        }
    }
    //提取文件名称中数字进行排序
    private static int extractNumber(String fileName) {
        Pattern pattern = Pattern.compile("^\\d+");
        Matcher matcher = pattern.matcher(fileName);
        if (matcher.find()) {
            return Integer.parseInt(matcher.group());
        }
        // 如果文件名不是以数字开头，则返回一个很大的数，使其排在最后
        return -1;
    }

    public static List<ImageUrlPair> generateFilePairs(Path inputPath, Path outputPath) throws IOException {
        String localPath = TaaisConfig.getProfile();
        Path localPathPath = Paths.get(localPath);


        // 替换路径
        return Files.list(inputPath)
            .filter(Files::isRegularFile)  // 只处理文件
            .map(path -> {
                String fileName = path.getFileName().toString();
                ImageUrlPair imageUrlPair = new ImageUrlPair();
                imageUrlPair.setInputUrl(inputPath.resolve(fileName).toString().replace(localPathPath.toString(), "/profile").replace("\\", "/"));
                imageUrlPair.setOutputUrl(outputPath.resolve(fileName).toString().replace(localPathPath.toString(), "/profile").replace("\\", "/"));
                return imageUrlPair;
            })
            .collect(Collectors.toList());
    }
    //新接口获取图片对序列，返回支持前端PreviewCompareImages组件的数据格式
    @GetMapping("/imageCompare/{task_id}")
    public CommonResult<List<ImageUrlPair>> getCompareImageSeq(@PathVariable("task_id") Long taskId) throws IOException {
        DataAugmentation dataAugmentation = dataAugmentationService.getById(taskId);
        Path inputPath = Paths.get(dataAugmentation.getInputPath());
        Path outputPath = Paths.get(dataAugmentation.getOutputPath());

        if ("多目标跟踪".equals(dataAugmentation.getTaskType())) {
            inputPath = inputPath.resolve("input");
            outputPath = outputPath.resolve("output");
        }
        List<ImageUrlPair> list;
        if (!Files.exists(inputPath) || !Files.isDirectory(inputPath)) {
            System.out.println("输入路径不存在或不是目录:" + inputPath.toString());
            return CommonResult.fail("输入路径不存在或不是目录:" + inputPath.toString());
        }
        if (!Files.exists(outputPath) || !Files.isDirectory(outputPath)) {
            System.out.println("输出路径不存在或不是目录:" + outputPath.toString());
            return CommonResult.fail("输出路径不存在或不是目录:" + outputPath.toString());
        }
        list = generateFilePairs(inputPath, outputPath);
        return  CommonResult.success(list);

    }
    @GetMapping("/compare/{task_id}")
    public ResponseEntity<Map<String,List<List<String>>>> getCompareImages(@PathVariable("task_id") Long taskId) {
        try {

            DataAugmentation dataAugmentation = dataAugmentationService.getById(taskId);
            Path inputPath = Paths.get(dataAugmentation.getInputPath());
            Path outputPath = Paths.get(dataAugmentation.getOutputPath());

            if ("多目标跟踪".equals(dataAugmentation.getTaskType())) {
                inputPath = inputPath.resolve("input");
                outputPath = outputPath.resolve("output");
            }
//            System.out.println("inputPath: " + inputPath.toString());
//            System.out.println("outputPath: " + outputPath.toString());
            Map<String, List<List<String>>> images = new HashMap<>();
            List<List<String>> stable_list = new ArrayList<>();
            List<List<String>> origin_list = new ArrayList<>();
            if (!Files.exists(inputPath) || !Files.isDirectory(inputPath)) {
                System.out.println("输入路径不存在或不是目录:" + inputPath.toString());
                ArrayList<String> strings = new ArrayList<>();
                strings.add("输入路径不存在或不是目录:" + inputPath.toString());
                origin_list.add(strings);
                images.put("error", origin_list);
                return ResponseEntity.status(500).body(images);
            }
            if (!Files.exists(outputPath) || !Files.isDirectory(outputPath)) {
                System.out.println("输出路径不存在或不是目录:" + outputPath.toString());
                ArrayList<String> strings = new ArrayList<>();
                strings.add("输出路径不存在或不是目录:" + outputPath.toString());
                stable_list.add(strings);
                images.put("error", stable_list);
                return ResponseEntity.status(500).body(images);
            }
            HashMap<String, Integer> compareNum = dataAugmentationService.getCompareNum(taskId);
            Integer integer = compareNum.get("inFileCount");
            for (int idx = 0; idx < integer; idx++) {

                ArrayList<String> stable = new ArrayList<>();
                ArrayList<String> origin = new ArrayList<>();
                Path imagePath = getImageAtPathIdx(inputPath, idx);  //按照自然排序获取索引为idx的imagePath
                List<String> outputFileList = new ArrayList<>(); //初始化结果文件list
                List<String> inputFileList = new ArrayList<>(); //初始化输入文件list

                String taskType = dataAugmentation.getTaskType();
                Path inputPath1 = null;
                Path outputPath1 = null;
                //图像拼接算法、目标毁伤评估有多张输入图片，则imagePath是个目录
                if (Files.isDirectory(imagePath)) {
//            if ("侦察图像拼接算法_sift".equals(taskType) || "侦察图像拼接算法_coordinate".equals(taskType) || "目标毁伤评估".equals(taskType)) {
                    String lastDirectoryName = imagePath.getFileName().toString();
                    inputPath1 = imagePath;
                    outputPath1 = outputPath.resolve(lastDirectoryName);  //得到算法输出的结果目录

                    if (Files.exists(outputPath1) && Files.isDirectory(outputPath1)) {
                        Stream<Path> outFilePathStream = Files.list(outputPath1);
                        // 检查流中是否有下一个元素
                        outputFileList = outFilePathStream
                            .filter(Files::isRegularFile)  // 只选择常规文件（排除子目录、排除图像拼接算法_sift输入中的txt文件）
                            .filter(path -> !path.toString().toLowerCase().endsWith(".txt")).map(path -> {
                                return path.getFileName().toString();
                            }).sorted().collect(Collectors.toList());
                        if ("目标毁伤评估".equals(taskType)) {
//                            QueryWrapper queryWrapper = new QueryWrapper();
//                            queryWrapper.eq("dict_type", "biz_data_augmentation");
//                            queryWrapper.eq("dict_label", "目标毁伤评估必须输出文件名称");
//                            List<SysDictData> list = sysDictDataService.list(queryWrapper);
                            String[] split = new String[]{"diffIm.jpg","largest_salient_region.jpg","final_region.jpg"};
//                            if (list.isEmpty()) {
//                                origin.add("未在数据字典中定义目标毁伤评估必须输出文件名称");
//                                origin_list.add(origin);
//                                images.put("error", origin_list);
//                                return ResponseEntity.status(500).body(images);
//                            } else {

                            for (String fileName: split) {
                                if (!outputFileList.contains(fileName)) {
                                    stable.add("输出目录不存在结果文件：" + fileName);
                                    stable_list.add(stable);
                                    images.put("error", stable_list);
                                    return ResponseEntity.status(500).body(images);
                                }
                            }

                            outputFileList.sort(Comparator.comparingInt(DataAugmentationController::extractNumber));
                            Map<String, Integer> indexMap = new HashMap<>();
                            for (int i = 0; i < split.length; i++) {
                                indexMap.put(split[i], i);
                            }
                            // 自定义比较器，基于indexMap中存储的位置信息
                            Comparator<String> customComparator = (o1, o2) -> {
                                Integer index1 = indexMap.get(o1);
                                Integer index2 = indexMap.get(o2);
                                // 如果o1或o2不在s数组中，则保持它们在原始列表中的相对顺序
                                if (index1 == null && index2 == null) {
                                    return 0;
                                } else if (index1 == null) {
                                    return 1;
                                } else if (index2 == null) {
                                    return -1;
                                }
                                return index1.compareTo(index2);
                            };
                            Collections.sort(outputFileList, customComparator);
                        }
                    } else {
                        System.out.println(taskType + "任务：" + dataAugmentation.getId() + "未创建结果目录！");
                        origin.add(taskType + "任务：" + dataAugmentation.getId() + "未创建结果目录！");
                        origin_list.add(origin);
                        images.put("error", origin_list);
                        return ResponseEntity.status(500).body(images);
                    }
                    Stream<Path> inputFilePathStream = Files.list(imagePath);
                    inputFileList = inputFilePathStream
                        .filter(Files::isRegularFile)  // 只选择常规文件（排除子目录、排除图像拼接算法_sift输入中的txt文件）
                        .filter(path -> !path.toString().toLowerCase().endsWith(".txt"))
                        .filter(path -> !path.toString().toLowerCase().endsWith(".json"))
                        .map(path -> {
                            return path.getFileName().toString();
                        }).sorted().collect(Collectors.toList());
                } else {
                    //算法输入是一对一时，输出文件名称和输入文件名称相同
                    inputPath1 = inputPath;
                    outputPath1 = outputPath;  //得到算法输出的结果目录
                    String inputFileName = imagePath.getFileName().toString();
                    inputFileList.add(inputFileName);
                    outputFileList.add(inputFileName);
                }

                if (inputFileList.isEmpty()) {
                    System.out.println(inputPath1.toString() + "：输入文件为空" );
                    origin.add(inputPath1.toString() + "：输入文件为空");
                    origin_list.add(origin);
                    images.put("error", origin_list);
                    return ResponseEntity.status(500).body(images);
                }
                if (outputFileList.isEmpty()) {
                    System.out.println(outputFileList.toString() + "：输出文件为空" );
                    stable.add(outputFileList.toString() + "：输入文件为空");
                    stable_list.add(origin);
                    images.put("error", stable_list);
                    return ResponseEntity.status(500).body(images);
                }
                String localPath = TaaisConfig.getProfile();
                Path localPathPath = Paths.get(localPath);


                for (String inputFileName: inputFileList) {
                    Path inputFilePath = inputPath1.resolve(inputFileName);
                    origin.add(inputFilePath.toString().replace(localPathPath.toString(), "/api/profile").replace("\\", "/"));
                }
                for (String outputFileName: outputFileList) {
                    Path outputFilePath = outputPath1.resolve(outputFileName);
                    stable.add(outputFilePath.toString().replace(localPathPath.toString(), "/api/profile").replace("\\", "/"));
                }
                origin_list.add(origin);
                stable_list.add(stable);

            }
            images.put("origin", origin_list);
            images.put("stable", stable_list);

            // 返回Map
            return ResponseEntity.ok(images);

        } catch (Exception e) {
            e.printStackTrace();
            return ResponseEntity.status(500).build();
        }
    }

    @PostMapping("/get_result")
    public CommonResult getResult(@Valid @RequestBody DataAugmentationResultBo dataAugmentationResultBo) {
        return dataAugmentationService.getResult(dataAugmentationResultBo);
    }

    @GetMapping("/start/{id}")
    public CommonResult start(@PathVariable("id") Long id) {
        return dataAugmentationService.start(id);
    }

    @GetMapping("/stop/{id}")
    public CommonResult stop(@PathVariable("id") Long id) {
        return dataAugmentationService.stop(id);
    }

    /**
     * 查询数据增强列表
     */
    @SaCheckPermission("demo:dataAugmentation:list")
    @GetMapping("/list")
    public CommonResult<PageResult<DataAugmentationVo>> list(DataAugmentationBo dataAugmentationBo) {
        if (dataAugmentationBo.getTaskType() == null) {
            return CommonResult.success(dataAugmentationService.selectPage1(dataAugmentationBo, "dataAugmentation"));
        }
        return CommonResult.success(dataAugmentationService.selectPage(dataAugmentationBo));
    }
    @SaCheckPermission("demo:dataAugmentation:list")
    @GetMapping("/imageMosaicList")
    public CommonResult<PageResult<DataAugmentationVo>> imageMosaicList(DataAugmentationBo dataAugmentationBo) {
        if (dataAugmentationBo.getTaskType() == null) {
            return CommonResult.success(dataAugmentationService.selectPage1(dataAugmentationBo, "imageMosaic"));
        }
        return CommonResult.success(dataAugmentationService.selectPage(dataAugmentationBo));
    }

    @SaCheckPermission("demo:dataAugmentation:list")
    @GetMapping("/targetDamageAcessList")
    public CommonResult<PageResult<DataAugmentationVo>> targetDamageAcessList(DataAugmentationBo dataAugmentationBo) {
        if (dataAugmentationBo.getTaskType() == null) {
            return CommonResult.success(dataAugmentationService.selectPage1(dataAugmentationBo, "targetDamageAcess"));
        }
        return CommonResult.success(dataAugmentationService.selectPage(dataAugmentationBo));
    }
    @SaCheckPermission("demo:dataAugmentation:list")
    @GetMapping("/targetTrackList")
    public CommonResult<PageResult<DataAugmentationVo>> targetTrackList(DataAugmentationBo dataAugmentationBo) {
        if (dataAugmentationBo.getTaskType() == null) {
            return CommonResult.success(dataAugmentationService.selectPage1(dataAugmentationBo, "targetTrack"));
        }
        return CommonResult.success(dataAugmentationService.selectPage(dataAugmentationBo));
    }
    /**
     * 导出数据增强列表
     */
    @SaCheckPermission("demo:dataAugmentation:export")
    @Log(title = "数据增强", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody String zipDirPath) {

        try {

            String zipFilePath = zipDirPath.substring(1, zipDirPath.length() - 1) + ".zip";
            System.out.println( zipFilePath);
            File zipFile = new File(zipFilePath);
            String fileName = zipFile.getName();
            // 设置响应头
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            response.setHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=" + fileName);
            // 读取ZIP文件并写入响应流
            try (FileInputStream fis = new FileInputStream(zipFilePath)) {
                IOUtils.copy(fis, response.getOutputStream());
            }
            // 刷新输出流
            response.flushBuffer();
        } catch (IOException e) {
            e.printStackTrace();
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        }
    }

    /**
     * 获取数据增强详细信息
     */
    @SaCheckPermission("demo:dataAugmentation:query")
    @GetMapping(value = "/{id}")
    public CommonResult<DataAugmentationVo> getInfo(@PathVariable Long id) {
        return CommonResult.success(dataAugmentationService.selectById(id));
    }

    /**
     * 新增数据增强
     */
    @SaCheckPermission("demo:dataAugmentation:add")
    @Log(title = "数据增强", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping
    public CommonResult<Void> add(@Validated @RequestBody DataAugmentationBo dataAugmentationBo) {

        boolean inserted = dataAugmentationService.insert(dataAugmentationBo);
        if (!inserted) {
            return CommonResult.fail("新增数据增强记录失败！");
        }
        return CommonResult.success();
    }

    /**
     * 修改数据增强
     */
    @SaCheckPermission("demo:dataAugmentation:edit")
    @Log(title = "数据增强", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping
    public CommonResult<Void> edit(@Validated @RequestBody DataAugmentationBo dataAugmentationBo) {
        Boolean updated = dataAugmentationService.update(dataAugmentationBo);
        if (!updated) {
            return CommonResult.fail("修改数据增强记录失败!");
        }
        return CommonResult.success();
    }

    /**
     * 删除数据增强
     */
    @SaCheckPermission("demo:dataAugmentation:remove")
    @Log(title = "数据增强", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public CommonResult<Void> remove(@PathVariable Long[] ids) {
        boolean deleted = dataAugmentationService.deleteByIds(ids);
        if (!deleted) {
            return CommonResult.fail("删除数据增强记录失败!");
        }
        return CommonResult.success();
    }

    private String readLogFileWithEncoding(Path path, java.nio.charset.Charset charset) throws IOException {
        try (BufferedReader reader = Files.newBufferedReader(path, charset)) {
            StringBuilder content = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                content.append(line).append("\n");
            }
            return content.toString();
        } catch (IOException e) {
            return null; // 读取失败，返回 null
        }
    }
    /**
     * 获取日志字符串
     */
    @SaCheckPermission("demo:dataAugmentation:query")
    @GetMapping(value = "/log/{id}")
    public CommonResult<String> getLog(@PathVariable Long id) {
        DataAugmentation byId = dataAugmentationService.getById(id);
        String filePath = byId.getAlgorithmPath();
        try{
            Path path = Paths.get(filePath);
            if (!Files.exists(path)) {
                return CommonResult.fail( "日志文件" + filePath + "不存在！");
            }

            String content = new String(Files.readAllBytes(path), StandardCharsets.UTF_8);

            return CommonResult.success(content, "获取日志成功");
        } catch (Exception e) {
            return CommonResult.fail("日志文件" + filePath + "读取失败！");
        }
    }


    /**
     * 获取算法结果指标
     */
    @SaCheckPermission("demo:dataAugmentation:query")
    @GetMapping(value = "/metric/{id}")
    public CommonResult<List<Metric>> getMetric(@PathVariable Long id) {
        DataAugmentation byId = dataAugmentationService.getById(id);
        String outputPath = byId.getOutputPath();
        Path path = Paths.get(outputPath, "result.json");
        try{

            if (!Files.exists(path)) {
                return CommonResult.fail( "算法指标文件" + path.toString() + "不存在！");
            }
            StringBuilder jsonContent = new StringBuilder();
            try (BufferedReader reader = new BufferedReader(new FileReader(path.toString()))) {
                String line;
                while ((line = reader.readLine()) != null) {
                    jsonContent.append(line);
                }
            }
            // 解析 JSON 数组
            JSONArray jsonArray = JSONArray.parseArray(jsonContent.toString());
            List<Metric> metricList = new ArrayList<>();
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject jsonObject = jsonArray.getJSONObject(i);
                Metric metric = new Metric(
                    jsonObject.getString("name"),
                    jsonObject.getString("value")
                );
                metricList.add(metric);
            }
            return CommonResult.success((metricList), "获取算法指标成功");
        } catch (Exception e) {
            return CommonResult.fail("算法指标文件" + path.toString() + "读取失败！");
        }
    }

    @SaCheckPermission("demo:dataAugmentation:query")
    @PostMapping(value = "/getTaskDictData")
    public CommonResult<List<TaskDictDataVo>> getTaskDictData(@RequestBody String module) {
        module = module.substring(1, module.length() - 1);  //去除字符串两把的引号
        ArrayList<TaskDictDataVo> taskDictDataVos = new ArrayList<>();
        //之前从数据字典中获取算法任务及其对应超参配置
//        QueryWrapper queryWrapper = new QueryWrapper();
//        queryWrapper.eq("dict_type", "biz_data_augmentation");
//        queryWrapper.eq("dict_label", "任务类型");
//        List<SysDictData> taskTypeList = sysDictDataService.list(queryWrapper);
//        for (SysDictData sysDictData: taskTypeList) {
//            String dictValue = sysDictData.getDictValue();
//            queryWrapper.clear();
//            queryWrapper.eq("dict_type", "biz_data_augmentation");
//            queryWrapper.eq("dict_label", dictValue + "超参配置");
//            List<SysDictData> hyparamList = sysDictDataService.list(queryWrapper);
//            if (hyparamList.isEmpty()) {
//                return CommonResult.fail("未在type为biz_data_augmentation的字典数据中设置" + dictValue + "的超参配置");
//            }
//            TaskDictDataVo taskDictDataVo = new TaskDictDataVo();
//            taskDictDataVo.setTaskType(dictValue);
//            taskDictDataVo.setHyperparameterConfiguration(hyparamList.get(0).getDictValue());
//            taskDictDataVos.add(taskDictDataVo);
//        }
        //现在从新建的通用算法配置表中获取
        //List<CommonAlgorithmConfig> list = commonAlgorithmConfigService.selectAll();
//        System.out.println("model" + module);
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("module", module);
        List<CommonAlgorithmConfig> list = commonAlgorithmConfigService.list(queryWrapper);
        if (list.isEmpty()) {
            return CommonResult.fail("通用算法配置表为空！");
        }
        for (CommonAlgorithmConfig commonAlgorithmConfig: list) {

            TaskDictDataVo taskDictDataVo = new TaskDictDataVo();
            taskDictDataVo.setTaskType(commonAlgorithmConfig.getAlgorithmName());
            taskDictDataVo.setHyperparameterConfiguration(commonAlgorithmConfig.getParameters());
            taskDictDataVos.add(taskDictDataVo);


        }
        return CommonResult.success(taskDictDataVos);

    }
    private static boolean hasSupportedExtension(String filePath, List<String> extensions) {
        for (String ext : extensions) {
            if (filePath.toLowerCase().endsWith(ext)) {
                return true;
            }
        }
        return false;
    }
    public static String findFirstVideoFile(String dirPath, List<String> extensions) {
        File dir = new File(dirPath);
        if (!dir.exists() || !dir.isDirectory()) {
            return null;
        }

        try (Stream<Path> stream = Files.list(Paths.get(dirPath))) {
            return stream
                .filter(Files::isRegularFile)
                .map(Path::toString)
                .filter(filePath -> hasSupportedExtension(filePath, extensions))
                .findFirst()
                .orElse(null);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    @SaCheckPermission("demo:dataAugmentation:query")
    @GetMapping(value = "/videoUrl/{id}")
    public CommonResult<VideoUrl> getVideoUrl(@PathVariable Long id) {
        DataAugmentation byId = dataAugmentationService.getById(id);
        String localPath = TaaisConfig.getProfile();
        List<String> extensions = Arrays.asList(".mp4");
        String firstInputVideoFile = findFirstVideoFile(byId.getInputPath(), extensions);
        if (firstInputVideoFile == null) {
            return CommonResult.fail(byId.getInputPath() + "目录下不存在输入视频文件!");
        }
        String firstOutputVideoFile = findFirstVideoFile(byId.getOutputPath(), extensions);
        if (firstOutputVideoFile == null) {
            return CommonResult.fail(byId.getOutputPath() + "目录下不存在输出视频文件!");
        }
        Path localPathPath = Paths.get(localPath);
        Path inputPath = Paths.get(firstInputVideoFile);
        Path outputPath = Paths.get(firstOutputVideoFile);

        // 替换路径
        String input = inputPath.toString().replace(localPathPath.toString(), "/api/profile").replace("\\", "/");
        String output = outputPath.toString().replace(localPathPath.toString(), "/api/profile").replace("\\", "/");
        VideoUrl videoUrl = new VideoUrl();
        videoUrl.setInput(input);
        videoUrl.setOutput(output);
        return  CommonResult.success(videoUrl);
    }

}
