package com.taais.biz.domain.bo;

import com.fasterxml.jackson.annotation.JsonAnySetter;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.taais.biz.domain.AlgorithmTaskTrack;
import com.taais.common.orm.core.domain.BaseEntity;
import io.github.linpeilie.annotations.AutoMapper;
import jakarta.validation.constraints.NotBlank;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 可辨识性分析总任务业务对象 algorithm_task_track
 *
 * @author wangkang
 * @date 2025-01-06
 */
@Data
@EqualsAndHashCode(callSuper = true)
@AutoMapper(target = AlgorithmTaskTrack.class, reverseConvertGenerate = false)
public class AlgorithmTaskTrackBo extends BaseEntity {
    /**
     * 主键ID
     */
    private Long id;

    /**
     * 任务名称
     */
    @NotBlank(message = "任务名称不能为空")
    private String name;

    /**
     * 任务状态
     */
    private String status;

    /**
     * 开始时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date startTime;

    /**
     * 结束时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date endTime;

    /**
     * 耗时
     */
    private Long costSecond;

    /**
     * 日志
     */
    private String log;

    /**
     * 备注
     */
    private String remarks;

    /**
     * 系统
     */
    private String system;

    // 将其他参数存入Map
    private Map<String, Object> otherParams = new HashMap<>();

    @JsonAnySetter
    public void addOtherParam(String name, Object value) {
        this.otherParams.put(name, value);
    }
}
